#####################################################################
# mail2fax.rc - configuration file for mail2fax.pl
# Last updated: Wed Aug 16 14:27:14 EDT 1995
# By: mail2fax.pl, Version 2.00
#--------------------------------------------------------------------
# Format:
#   Any line starting with '#' is a comment
#   All legal entries have three items separated by a space(s)
#   Several sections (see below for explanation)
#====================================================================
# The program to spool a FAX
# It has to accept a recipient via an option '-D', then the fax
#  number and lastly the fax text as arguments.
=FaxSpool=  /usr/local/bin/faxspool   
#====================================================================
# The command to send a mail with.
# It has to expect a complete mail message at it's standard input.
=Mailer=    /usr/lib/sendmail -t  
#====================================================================
# The billing option
# By default this is on, a value of 0 disables it
=Billing=   1
#====================================================================
# The debug option
# By default this is off, any value >0 enables it
=Debug=     0
#====================================================================
# The name for an unknown recipient
# By default this is "unknown"
=Unknown=   unknown
#====================================================================
# The FAX administrator
# By default this is faxadmin
=FaxAdmin=  faxadmin
#====================================================================
# The phone zones for billing purposes
# The cheapest is 1, the most expensive one is 6
# Each zone defines the phone number prefix and the price per FAX in
#  1/100 of the currency
# Zone 1 - local calls (no special prefix)
# Zone 2 - calls within the same area code (long-distance within LATA)
# Zone 3 - calls to adjacent area code (often same price as LATA)
# Zone 4 - calls to area codes in the state
# Zone 5 - calls to area codes out-of-state
# Zone 6 - calls out of the country
=Zone=1   -1  0
=Zone=2   1716  50
=Zone=3   1315  50
=Zone=4   1  100
=Zone=5   1  100
=Zone=6   011  350
#====================================================================
# The user list
# Each entry consists of the user-ID, the password, a discount factor,
#  and the remaining credits in 1/100 of the currency
=User=  test2  test  1   10000
=User=  test1  test  .8   1000
=User=  test  test  0   1
#====================================================================
